#!/usr/bin/env python
import os

# version = (3, 5, 3)
version = (3, 6, 3)
# version = (3, 4, 2)
major_minor = "%s.%s" % version[0:2]
vers_str = "%s.%s.%s" % version
os.chdir("/home/rocky/.pyenv/versions/%s/lib/python%s/" % (vers_str, major_minor))

tests = """
    test_concurrent_futures test_configparser test_contains
    test_contextlib test_copy test_copyreg test_coroutines
    test_cprofile test_crashers test_crypt test_csv test_ctypes
    test_curses test_datetime test_dbm test_dbm_dumb test_dbm_gnu
    test_dbm_ndbm test_decimal test_decorators test_defaultdict
    test_deque test_descr test_descrtut test_devpoll test_dict_version
    test_dictcomps test_dictviews test_difflib test_dis test_distutils
    test_docxmlrpc test_dtrace test_dummy_thread test_dummy_threading
    test_dynamic test_dynamicclassattribute test_eintr test_email
    test_ensurepip test_enum test_enumerate test_eof test_epoll
    test_errno test_exception_hierarchy test_exception_variations
    test_extcall test_faulthandler test_fcntl test_file
    test_file_eintr test_filecmp test_fileinput test_fileio
    test_finalization test_float test_flufl test_fnmatch test_fork1
    test_format test_fractions test_frame test_fstring test_ftplib
    test_funcattrs test_functools test_future test_future3
    test_future4 test_future5 test_gc test_gdb test_generator_stop
    test_generators test_genericpath test_genexps test_getargs2
    test_getopt test_getpass test_gettext test_glob test_global
    test_grp test_gzip test_hash test_hashlib test_heapq test_hmac
    test_html test_htmlparser test_http_cookiejar test_http_cookies
    test_httplib test_httpservers test_idle test_imaplib test_imghdr
    test_imp test_import test_importlib test_index test_inspect
    test_int test_int_literal test_io test_ioctl test_ipaddress
    test_isinstance test_iter test_iterlen test_itertools test_json
    test_keyword test_keywordonlyarg test_kqueue test_largefile
    test_lib2to3 test_linecache test_list test_listcomps test_locale
    test_logging test_long test_longexp test_lzma test_macpath
    test_macurl2path test_mailbox test_mailcap test_marshal test_math
    test_memoryio test_memoryview test_metaclass test_mimetypes
    test_minidom test_mmap test_module test_modulefinder test_msilib
    test_multibytecodec test_multiprocessing_fork
    test_multiprocessing_forkserver test_multiprocessing_main_handling
    test_multiprocessing_spawn test_netrc test_nis test_nntplib
    test_normalization test_ntpath test_numeric_tower test_openpty
    test_operator test_optparse test_ordered_dict test_os
    test_ossaudiodev test_osx_env test_parser test_pathlib test_pdb
    test_peepholer test_pickle test_pickletools test_pipes test_pkg
    test_pkgimport test_pkgutil test_platform test_plistlib test_poll
    test_popen test_poplib test_posix test_posixpath test_pow
    test_pprint test_print test_profile test_property test_pstats
    test_pty test_pulldom test_pwd test_py_compile test_pyclbr
    test_pydoc test_pyexpat test_queue test_quopri test_raise
    test_random test_range test_re test_readline test_regrtest
    test_reprlib test_resource test_richcmp test_rlcompleter
    test_robotparser test_runpy test_sax test_sched test_scope
    test_script_helper test_secrets test_select test_selectors
    test_set test_setcomps test_shelve test_shlex test_shutil
    test_signal test_site test_slice test_smtpd test_smtplib
    test_smtpnet test_sndhdr test_socket test_socketserver test_sort
    test_source_encoding test_spwd test_sqlite test_ssl test_startfile
    test_stat test_statistics test_strftime test_string
    test_string_literals test_stringprep test_strptime test_strtod
    test_struct test_structmembers test_structseq test_subclassinit
    test_subprocess test_sunau test_sundry test_super test_symbol
    test_symtable test_syntax test_sys test_sys_setprofile
    test_sys_settrace test_sysconfig test_syslog test_tarfile test_tcl
    test_telnetlib test_tempfile test_textwrap test_thread
    test_threaded_import test_threadedtempfile test_threading
    test_threading_local test_threadsignals test_time test_timeit
    test_timeout test_tix test_tk test_tokenize test_tools test_trace
    test_traceback test_tracemalloc test_ttk_guionly test_ttk_textonly
    test_tuple test_turtle test_typechecks test_typing test_ucn
    test_unary test_unicode test_unicode_file
    test_unicode_file_functions test_unicode_identifiers
    test_unicodedata test_univnewlines test_unpack test_unpack_ex
    test_urllib test_urllib2 test_urllib2_localnet test_urllib2net
    test_urllib_response test_urllibnet test_urlparse test_userdict
    test_userlist test_userstring test_utf8source test_uu test_uuid
    test_venv test_wait3 test_wait4 test_warnings test_wave
    test_weakref test_weakset test_webbrowser test_winconsoleio
    test_winreg test_winsound test_with test_wsgiref test_xdrlib
    test_xml_dom_minicompat test_xml_etree test_xml_etree_c
    test_xmlrpc test_xmlrpc_net test_yield_from test_zipapp
    test_zipfile test_zipfile64 test_zipimport test_zipimport_support
    test_zlib
""".split()

# from test.libregrtest.main import Regrtest as Regrtest
from test.libregrtest import main as main
# r = Regrtest()
# tests = r.find_tests(tests)
# print(r)
# main(tests=['test_copy'])
main()
